@@ -4,122 +4,147 @@ use strict;
 use warnings;
 BEGIN { warnings->import(FATAL => 'layer') if $] >= 5.008; }
 
-our $VERSION = '1.0001';
+our $VERSION = '1.0101';
 
 use File::Basename qw(basename);
 use Carp qw(croak);
 use Fcntl ();
 use Errno ();
-use Exporter qw(import);
+use Exporter (); BEGIN { our @ISA = 'Exporter'; }
 
-our @EXPORT_OK = qw(fopen fopen_nothrow fsysopen fsysopen_nothrow);
+our @EXPORT_OK = qw(
+    fopen    fopen_nothrow
+    fsysopen fsysopen_nothrow
+    fopendir fopendir_nothrow
+);
 
 sub _mode {
-	map { $_ => $_[0] } @_
+    map { $_ => $_[0] } @_
 }
 
 my %modemap = (
-	_mode(qw[<   r ]),
-	_mode(qw[>   w ]),
-	_mode(qw[>>  a ]),
-	_mode(qw[+<  r+]),
-	_mode(qw[+>  w+]),
-	_mode(qw[+>> a+]),
+    _mode(qw[<   r ]),
+    _mode(qw[>   w ]),
+    _mode(qw[>>  a ]),
+    _mode(qw[+<  r+]),
+    _mode(qw[+>  w+]),
+    _mode(qw[+>> a+]),
 );
 
 sub _open {
-	my ($func, $file, $mode, $layers) = @_;
-	@_ < 2 and croak "Not enough arguments for $func";
-	@_ > 4 and croak "Too many arguments for $func";
-	$mode = '<' if !defined $mode;
+    my ($func, $file, $mode, $layers) = @_;
+    @_ < 2 and croak "Not enough arguments for $func";
+    @_ > 4 and croak "Too many arguments for $func";
+    $mode = '<' if !defined $mode;
 
-	my $binary = $mode =~ s/(?<=.)b//;
-	my $emode = $modemap{$mode} or croak "Unknown $func() mode '$mode'";
+    my $binary = $mode =~ s/(?<=.)b//;
+    my $emode = $modemap{$mode} or croak "Unknown $func() mode '$mode'";
 
-	if ($file =~ /\0/) {
-		$! = Errno::ENOENT() if exists &Errno::ENOENT;
-		return undef;
-	}
+    if ($file =~ /\0/) {
+        $! = Errno::ENOENT() if exists &Errno::ENOENT;
+        return undef;
+    }
 
-	unless (defined $layers) {
-		# grab our caller's 'use open' settings
-		my $hints = (caller 1)[10];
-		my $key = $emode =~ />/ ? 'open>' : 'open<';
-		$layers = $hints->{$key};
-	}
+    unless (defined $layers) {
+        # grab our caller's 'use open' settings
+        my $hints = (caller 1)[10];
+        my $key = $emode =~ />/ ? 'open>' : 'open<';
+        $layers = $hints->{$key};
+    }
 
-	open my $fh, $emode . (defined $layers ? " $layers" : ''), $file or return undef;
-	binmode $fh if $binary;
-	$fh
+    open my $fh, $emode . (defined $layers ? " $layers" : ''), $file or return undef;
+    binmode $fh if $binary;
+    $fh
 }
 
 sub fopen_nothrow {
-	_open('fopen_nothrow', @_)
+    _open('fopen_nothrow', @_)
 }
 
 my $prog = basename $0;
 
 sub fopen {
-	_open('fopen', @_) || die "$prog: $_[0]: $!\n"
+    _open('fopen', @_) || die "$prog: $_[0]: $!\n"
 }
 
 sub _sysopen {
-	my ($func, $file, $mode, $flags) = @_;
-	@_ < 3 and croak "Not enough arguments for $func";
-	@_ > 4 and croak "Too many arguments for $func";
+    my ($func, $file, $mode, $flags) = @_;
+    @_ < 3 and croak "Not enough arguments for $func";
+    @_ > 4 and croak "Too many arguments for $func";
 
-	my $emode =
-		$mode eq 'r'  ? Fcntl::O_RDONLY() :
-		$mode eq 'w'  ? Fcntl::O_WRONLY() :
-		$mode eq 'rw' ? Fcntl::O_RDWR()   :
-		croak "Unknown $func() mode '$mode'"
-	;
-	$flags = {} if !defined $flags;
+    my $emode =
+        $mode eq 'r'  ? Fcntl::O_RDONLY() :
+        $mode eq 'w'  ? Fcntl::O_WRONLY() :
+        $mode eq 'rw' ? Fcntl::O_RDWR()   :
+        croak "Unknown $func() mode '$mode'"
+    ;
+    $flags = {} if !defined $flags;
 
-	my $perms = 0;
+    my $perms = 0;
 
-	for my $k (keys %$flags) {
-		my $v = !!$flags->{$k};
-		$emode |=
-			$k eq 'creat' ?
-				defined $flags->{$k} ? do {
-					$perms = $flags->{$k};
-					Fcntl::O_CREAT()
-				} :
-				0
-			:
-			$k eq 'append'    ? $v && Fcntl::O_APPEND()    :
-			$k eq 'async'     ? $v && Fcntl::O_ASYNC()     :
-			$k eq 'direct'    ? $v && Fcntl::O_DIRECT()    :
-			$k eq 'directory' ? $v && Fcntl::O_DIRECTORY() :
-			$k eq 'excl'      ? $v && Fcntl::O_EXCL()      :
-			$k eq 'noatime'   ? $v && Fcntl::O_NOATIME()   :
-			$k eq 'noctty'    ? $v && Fcntl::O_NOCTTY()    :
-			$k eq 'nofollow'  ? $v && Fcntl::O_NOFOLLOW()  :
-			$k eq 'nonblock'  ? $v && Fcntl::O_NONBLOCK()  :
-			$k eq 'sync'      ? $v && Fcntl::O_SYNC()      :
-			$k eq 'trunc'     ? $v && Fcntl::O_TRUNC()     :
-			croak "Unknown $func() flag '$k'"
-		;
-	}
+    for my $k (keys %$flags) {
+        my $v = !!$flags->{$k};
+        $emode |=
+            $k eq 'creat' ?
+                defined $flags->{$k} ? do {
+                    $perms = $flags->{$k};
+                    Fcntl::O_CREAT()
+                } :
+                0
+            :
+            $k eq 'append'    ? $v && Fcntl::O_APPEND()    :
+            $k eq 'async'     ? $v && Fcntl::O_ASYNC()     :
+            $k eq 'direct'    ? $v && Fcntl::O_DIRECT()    :
+            $k eq 'directory' ? $v && Fcntl::O_DIRECTORY() :
+            $k eq 'excl'      ? $v && Fcntl::O_EXCL()      :
+            $k eq 'noatime'   ? $v && Fcntl::O_NOATIME()   :
+            $k eq 'noctty'    ? $v && Fcntl::O_NOCTTY()    :
+            $k eq 'nofollow'  ? $v && Fcntl::O_NOFOLLOW()  :
+            $k eq 'nonblock'  ? $v && Fcntl::O_NONBLOCK()  :
+            $k eq 'sync'      ? $v && Fcntl::O_SYNC()      :
+            $k eq 'trunc'     ? $v && Fcntl::O_TRUNC()     :
+            croak "Unknown $func() flag '$k'"
+        ;
+    }
 
-	if ($file =~ /\0/) {
-		$! = Errno::ENOENT() if exists &Errno::ENOENT;
-		return undef;
-	}
+    if ($file =~ /\0/) {
+        $! = Errno::ENOENT() if exists &Errno::ENOENT;
+        return undef;
+    }
 
-	sysopen my $fh, $file, $emode, $perms or return undef;
-	$fh
+    sysopen my $fh, $file, $emode, $perms or return undef;
+    $fh
 }
 
 sub fsysopen_nothrow {
-	_sysopen('fsysopen_nothrow', @_)
+    _sysopen('fsysopen_nothrow', @_)
 }
 
 sub fsysopen {
-	_sysopen('fsysopen', @_) || die "$prog: $_[0]: $!\n"
+    _sysopen('fsysopen', @_) || die "$prog: $_[0]: $!\n"
 }
 
+sub _opendir {
+    my ($func, $dir) = @_;
+    @_ < 2 and croak "Not enough arguments for $func";
+    @_ > 2 and croak "Too many arguments for $func";
+
+    if ($dir =~ /\0/) {
+        $! = Errno::ENOENT() if exists &Errno::ENOENT;
+        return undef;
+    }
+
+    opendir my $dh, $dir or return undef;
+    $dh
+}
+
+sub fopendir_nothrow {
+    _opendir('fopendir_nothrow', @_)
+}
+
+sub fopendir {
+    _opendir('fopendir', @_) || die "$prog: $_[0]: $!\n"
+}
 
 'ok'
 
@@ -127,11 +152,15 @@ __END__
 
 =head1 NAME
 
-File::Open - wrap open/sysopen and give them a nice and simple interface
+File::Open - wrap open/sysopen/opendir and give them a nice and simple interface
 
 =head1 SYNOPSIS
 
- use File::Open qw(fopen fopen_nothrow fsysopen fsysopen_nothrow);
+ use File::Open qw(
+     fopen    fopen_nothrow
+     fsysopen fsysopen_nothrow
+     fopendir fopendir_nothrow
+ );
 
  my $fh = fopen $file;
  my $fh = fopen $file, $mode;
@@ -147,6 +176,10 @@ File::Open - wrap open/sysopen and give them a nice and simple interface
  my $fh = fsysopen_nothrow $file, $mode or die "$0: $file: $!\n";
  my $fh = fsysopen_nothrow $file, $mode, \%flags or die "$0: $file: $!\n";
 
+ my $dh = fopendir $dir;
+
+ my $dh = fopendir_nothrow $dir or die "$0: $dir: $!\n";
+
 =head1 EXAMPLES
 
  sub slurp {
@@ -163,11 +196,17 @@ File::Open - wrap open/sysopen and give them a nice and simple interface
  my $lock_fh = fsysopen $lock_file, 'w', { creat => 0644 };
  flock $lock_fh, LOCK_EX or die "$0: $lock_file: $!\n";
 
+ my @entries = readdir fopendir '.';
+
 =head1 DESCRIPTION
 
-This module provides convenience wrappers around L<open|perlfunc/open> and
-L<sysopen|perlfunc/sysopen> for opening simple files. Nothing is exported by
-default; you have to specify every function you want to import explicitly.
+This module provides convenience wrappers around
+L<C<open>|perlfunc/open FILEHANDLE,EXPR> and
+L<C<sysopen>|perlfunc/sysopen FILEHANDLE,FILENAME,MODE>
+for opening simple files and a wrapper around
+L<C<opendir>|perlfunc/opendir DIRHANDLE,EXPR> for opening directories. Nothing
+is exported by default; you have to specify every function you want to import
+explicitly.
 
 =head2 Functions
 
@@ -216,14 +255,15 @@ Open the file for appending (like C<'a'>) but also allow reads.
 =back
 
 In addition you can append a C<'b'> to each of the mode strings listed above.
-This will cause L<binmode|perlfunc/binmode> to be called on the filehandle.
+This will cause L<C<binmode>|perlfunc/binmode FILEHANDLE, LAYER> to be called
+on the filehandle.
 
 If you don't specify a MODE, it defaults to C<'r'>.
 
 If you pass LAYERS, C<fopen> will combine it with the open mode in the
-underlying L<open|perlfunc/open> call. This gives you greater control than the
-simple C<'b'> in MODE (which is equivalent to passing C<:raw> as LAYERS). For
-example, to read from a UTF-8 file:
+underlying L<C<open>|perlfunc/open FILEHANDLE,EXPR> call. This gives you
+greater control than the simple C<'b'> in MODE (which is equivalent to passing
+C<:raw> as LAYERS). For example, to read from a UTF-8 file:
 
   my $fh = fopen $file, 'r', ':encoding(UTF-8)';
   # does
@@ -240,7 +280,7 @@ encoding names, respectively.
 If you don't pass LAYERS, C<fopen> will use the default layers set via
 C<use open ...>, if any (see L<open>). Default layers aren't supported on old
 perls (i.e. anything before 5.10.0); on those you'll have to pass an explicit
-LAYERS argument.
+LAYERS argument if you want to use encodings.
 
 =item fopen_nothrow FILE
 
@@ -255,7 +295,8 @@ returns C<undef>.
 
 =item fsysopen FILE, MODE, FLAGS
 
-Uses the more low-level interface of L<sysopen|perlfunc/sysopen> to open FILE.
+Uses the more low-level interface of
+L<C<sysopen>|perlfunc/sysopen FILEHANDLE,FILENAME,MODE> to open FILE.
 If it succeeds, it returns a filehandle; if it fails, it throws an exception of
 the form C<"$program: $filename: $!\n">.
 
@@ -268,7 +309,7 @@ hash keys are strings (specifying the flag) and the values are booleans
 (indicating whether the flag should be off (default) or on) - with one
 exception. The exception is the C<'creat'> flag; if set, its value must be a
 number that specifies the permissions of the newly created file. See
-L<perlfunc/umask> for details.
+L<perlfunc/umask EXPR> for details.
 
 The following flags are recognized:
 
@@ -308,9 +349,11 @@ try to specify a non-existent flag.
 
 =item fsysopen_nothrow FILE, MODE, FLAGS
 
-Works exactly like L<fsysopen|/"fsysopen FILE, MODE"> but if the sysopen fails
+Works exactly like L<C<fsysopen>|/fsysopen FILE, MODE> but if the sysopen fails
 it simply returns C<undef>.
 
+=item fopendir DIR
+
 =back
 
 =head2 Methods
@@ -339,9 +382,10 @@ method calls instead of functions:
 
 =head1 SEE ALSO
 
-L<perlfunc/open>,
-L<perlfunc/binmode>,
-L<perlfunc/sysopen>,
+L<perlfunc/open FILEHANDLE,EXPR>,
+L<perlfunc/binmode FILEHANDLE, LAYER>,
+L<perlfunc/sysopen FILEHANDLE,FILENAME,MODE>,
+L<perlfunc/opendir DIRHANDLE,EXPR>,
 L<perlopentut>,
 L<IO::Handle>,
 L<Fcntl>,
@@ -353,7 +397,7 @@ Lukas Mai, C<< <l.mai at web.de> >>
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2011, 2013 Lukas Mai.
+Copyright 2011, 2013, 2016 Lukas Mai.
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of either: the GNU General Public License as published
